//
//  AppDelegate.m
//  airinvestigazioni
//
//  Created by Andrea della Porta on 09/03/15.
//  Copyright (c) 2015 Ermes Italia. All rights reserved.
//

#import "AppDelegate.h"
#import "ViewController.h"
#import "CustomData.h"
#import "ChatViewController.h"
#import "LoginViewController.h"
#import "NotifiesListViewController.h"

@interface AppDelegate ()

@property UIView *splashView;
@property UIViewController *modalViewController;

@end

@implementation AppDelegate

@synthesize receivedData;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.splashView = [[[NSBundle mainBundle] loadNibNamed:@"LaunchScreen" owner:self options:nil] objectAtIndex:0];
    
    if(self.splashView)
    {
#if false
        self.modalViewController = [[UIViewController alloc] init];
        self.modalViewController.view = self.splashView;
        [self.window.rootViewController presentViewController:self.modalViewController animated:NO completion:nil];
        [self performSelector:@selector(hideSplash) withObject:nil afterDelay:SPLASHSCREEN_TIMEOUT];
#else
        [self.window addSubview:self.splashView];
        [self.window bringSubviewToFront:self.splashView];
     #if false
        [NSThread detachNewThreadSelector:@selector(waitForSplash:) toTarget:self withObject:nil];
     #else
        [NSThread sleepForTimeInterval:SPLASHSCREEN_TIMEOUT];
        [self.splashView removeFromSuperview];
     #endif
#endif
    }
    
    NSString *errorDesc = nil;
    NSPropertyListFormat format;
    NSString *plistPath;
    NSString *rootPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                              NSUserDomainMask, YES) objectAtIndex:0];
    plistPath = [rootPath stringByAppendingPathComponent:@"data.plist"];
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistPath]) {
        plistPath = [[NSBundle mainBundle] pathForResource:@"data" ofType:@"plist"];
    }
    NSData *plistXML = [[NSFileManager defaultManager] contentsAtPath:plistPath];
    NSDictionary *temp = (NSDictionary *)[NSPropertyListSerialization
                                          propertyListFromData:plistXML
                                          mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                          format:&format
                                          errorDescription:&errorDesc];
    BOOL isRegistered = NO;
    BOOL dismissRegistrationAlert = NO;
    if (!temp) {
        NSLog(@"Error reading plist: %@, format: %d", errorDesc, format);
    }
    else
    {
        NSString *username = [temp objectForKey:@"username"];
        [ChatViewController usernameSet:username];
        NSString *pwd = [temp objectForKey:@"pwd"];
        [ChatViewController pwdSet:pwd];
        isRegistered = [[temp objectForKey:@"isRegistered"] boolValue];
        [LoginViewController registerSet:isRegistered];
        dismissRegistrationAlert = [[temp objectForKey:@"dismiss"] boolValue];
        [ViewController dismissSet:dismissRegistrationAlert];
    }
    
    NSString* vcID;
    if(!PUSHER_USE_REGISTRATION)
    {
        vcID = @"homepageID";
    }
    else
    {
        if(!isRegistered)
            vcID = @"loginID";
        else vcID = @"homepageID";
    }

    //NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    
    self.window = [[UIWindow alloc] initWithFrame:UIScreen.mainScreen.bounds];
    
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    
    UIViewController *viewController = [storyboard instantiateViewControllerWithIdentifier:vcID];
    
    self.window.rootViewController = viewController;
    [self.window makeKeyAndVisible];

    if(IBA_ENABLED)
    {
        // Override point for customization after application launch.
        if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 8.0)
        {
            UIUserNotificationSettings *settings =
            [UIUserNotificationSettings settingsForTypes:UIUserNotificationTypeAlert |
             UIUserNotificationTypeBadge |
             UIUserNotificationTypeSound
                                              categories:nil];
            [[UIApplication sharedApplication] registerUserNotificationSettings:settings];
            [[UIApplication sharedApplication] registerForRemoteNotifications];
        }
        else
        {
            [[UIApplication sharedApplication] registerForRemoteNotificationTypes:
             UIRemoteNotificationTypeAlert |
             UIRemoteNotificationTypeBadge |
             UIRemoteNotificationTypeSound];
        }
        
        NSDictionary *notification = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
        if (notification != nil)
        {
            self.showNotifies = YES;
        } else {
            //NSLog(@"Did not receive a remote notification.");
        }
    }

    return YES;
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    //NSLog(@"Received response: %@", response);
    
    [receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    //NSLog(@"Received %d bytes of data", [data length]);
    //NSLog([[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]);
    [receivedData appendData:data];
    //NSLog(@"Received data is now %d bytes", [receivedData length]);
    
}

- (void)hideSplash
{
    [[self modalViewController] dismissViewControllerAnimated:YES completion:nil];
}

-(void)waitForSplash:(id)obj
{
    [NSThread sleepForTimeInterval:SPLASHSCREEN_TIMEOUT];
    [self.splashView removeFromSuperview];
}

//[splashView release];
- (BOOL) pushNotificationOnOrOff
{
    if ([UIApplication instancesRespondToSelector:@selector(isRegisteredForRemoteNotifications)]) {
        return ([[UIApplication sharedApplication] isRegisteredForRemoteNotifications]);
    } else {
        UIRemoteNotificationType types = [[UIApplication sharedApplication] enabledRemoteNotificationTypes];
        return (types & UIRemoteNotificationTypeAlert);
    }
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
    
    if(application.applicationState == UIApplicationStateActive) {
        self.alertArrived = YES;
    } else {
        self.showNotifies = YES;//or set alertArrived here if you want a background app to ask first
    }
    [[NSNotificationCenter defaultCenter] postNotificationName:@"remoteNotify" object:self userInfo:userInfo];
    
    //NSLog(userInfo.description);
    
    completionHandler(UIBackgroundFetchResultNewData);
}

/*
 #ifdef __IPHONE_8_0
 - (void)application:(UIApplication *)application   didRegisterUserNotificationSettings:   (UIUserNotificationSettings *)notificationSettings
 {
 //register to receive notifications
 [application registerForRemoteNotifications];
 }
 
 - (void)application:(UIApplication *)application handleActionWithIdentifier:(NSString   *)identifier forRemoteNotification:(NSDictionary *)userInfo completionHandler:(void(^)())completionHandler
 {
 //handle the actions
 if ([identifier isEqualToString:@"declineAction"]){
 }
 else if ([identifier isEqualToString:@"answerAction"]){
 }
 }
 #endif
*/

- (void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)deviceToken
{
    self.token_string = [[[[deviceToken description]    stringByReplacingOccurrencesOfString:@"<"withString:@""]
                                stringByReplacingOccurrencesOfString:@">" withString:@""]
                               stringByReplacingOccurrencesOfString: @" " withString: @""];
    //NSLog(@"My token is: %@", self.token_string);
    
    [self sendToken:self.token_string];
}

- (void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error
{
    //NSLog(@"Failed to get token, error: %@", error);
  
#if defined(IBATEST) || TARGET_IPHONE_SIMULATOR
    if(TARGET_IPHONE_SIMULATOR)
        self.token_string = TEST_TOKEN;
 
    [self sendToken:self.token_string];
 #endif
}

-(void)sendToken:(NSString*)token
{
    NSDictionary* bodyParameters = @{
                                    
                                #ifdef IBATEST
                                     @"device_token": TEST_TOKEN
                                #else
                                     @"device_token": ((!token)?@"unregistered":token)
                                #endif
                                     };
    
    //if([InfoPageViewController checkConnectivity])
    {
        HTTP *http = [[HTTP alloc] init];
        http.delegate = self;
        
        [http post:DEVICE_TOKEN_URL withParams:bodyParameters];
    }
}
  
- (void)didFinishDownload:(NSString *)data
{
    //NSString *data = [[NSString alloc] initWithData:receivedData encoding:NSASCIIStringEncoding];
    
    NSDictionary* contentArray;
    NSError* error;
    NSData *encData = [data dataUsingEncoding:NSUTF8StringEncoding];
    contentArray = [NSJSONSerialization JSONObjectWithData:encData options:kNilOptions error:&error];
    if (error != nil)
    {
        NSLog(@"%@", [error localizedDescription]);
        return;
    }
    
    if(contentArray != nil)
    {
        /*for(NSDictionary *itemArray in contentArray)
         {
         }*/
        
        if([contentArray objectForKey:@"errors"])
        {
            NSLog(@"An error occurred");
            NSString *errorStr = @"Unknown Error";
#if true
            NSDictionary* itemArray = [contentArray objectForKey:@"errors"];
            if(itemArray != nil && [itemArray count] > 0)
            {
                NSString *key = [[itemArray allKeys] objectAtIndex:0];
                NSString *str = [[itemArray objectForKey:key] objectAtIndex:0];
                errorStr = str;
            }
            
            UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Error!" message:errorStr delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
            [alertView show];
#endif
            return;
        }
        else
        {
            NSDictionary* itemArray = contentArray;//[contentArray objectForKey:@"user"];
            
            //NSString *email = [itemArray objectForKey:@"email"];
            [LoginViewController codeSet:[itemArray objectForKey:@"code"]];
            
            LoginViewController *lvc =  [[LoginViewController alloc] init];
            if([LoginViewController registerGet] /*&& ![LoginViewController loggedInGet]*/)
                [lvc Login:[ChatViewController usernameGet] withPwd:[ChatViewController pwdGet] shouldGoToHomepage:NO];
            
            [[NSNotificationCenter defaultCenter] postNotificationName:@"remoteNotify" object:self userInfo:nil];
            AppDelegate *delegate = [UIApplication sharedApplication].delegate;
            UIViewController *uivc = [ViewController visibleViewController];
            if(uivc && [uivc isKindOfClass:[ViewController class]])
                if(((ViewController*)uivc).innerViewController)
                    [((ViewController*)uivc).innerViewController actionrefresh:nil];
        }
    }
}

- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
