//
//  ChatViewController.m
//
//  Created by Andrea della Porta on 11/03/15.
//  Copyright (c) 2015 Ermes Italia. All rights reserved.
//

#import "ChatViewController.h"
#import "InfoPageViewController.h"
#import "CustomData.h"
#import "ViewController.h"
#import "NotifiesListViewController.h"
#import "AppDelegate.h"
#import "CustomData.h"
#import "MessageItem.h"
#import <QuartzCore/QuartzCore.h>
#import <MobileCoreServices/MobileCoreServices.h>
#import "UserDataTapGestureRecognizer.h"
#import "LoginViewController.h"
#import <AVFoundation/AVFoundation.h>

#define HISTORY_OFFSET  1
#define USE_PUSHER_PRESENCE_CHANNEL 1

@interface ChatViewController ()<PTPusherDelegate>
{
    PTPusher *_client;
}

@property (weak, nonatomic) IBOutlet UIView *containerView;
@property (weak, nonatomic) IBOutlet UIView *blackRibbon;
@property (weak, nonatomic) IBOutlet UIButton *back;
@property NSString *channelName;
@property NSURLConnection *site_connection;
@property NSString *downloadedData;
@property NSDateFormatter *dateFormat;
@property NSDateFormatter *formatterMsg;
@property NSMutableArray *notifyList;
@property BOOL shouldReconnect;
@property NSDictionary *viewsDictionary;
@property NSArray *constraint_H;
#ifdef USE_PUSHER_PRESENCE_CHANNEL
    @property PTPusherPresenceChannel *channel;
#else
    @property PTPusherPrivateChannel *channel;
#endif
@property UIImage *playIcon;

@property (weak, nonatomic) IBOutlet UIButton *addImageButton;
@property (weak, nonatomic) IBOutlet UIButton *historyButton;
@property (weak, nonatomic) IBOutlet UIButton *sendButton;
@property (weak, nonatomic) IBOutlet UILabel *blackR;
@property (weak, nonatomic) IBOutlet UIImageView *logo;
@property (weak, nonatomic) IBOutlet UIView *mainView;

@property UIImage *tickImage;

@end

static NSString *username;
static NSString *pwd;


UITextView *activeField;
CGFloat oldActiveYoffset;
int historyPage, lastHistoryPage, historyOffset;
#define TEXT_VIEW_HEIGHT_MULT 0.15
BOOL reconnectOnShow = NO;

@implementation ChatViewController

@synthesize player;


#define IMAGE_PINCH_ZOOM_ENABLED2   YES


+ (NSString *)usernameGet
{
    return username;
}

+ (void)usernameSet:(NSString *)name
{
    username = name;
}

+ (NSString *)pwdGet
{
    return pwd;
}

+ (void)pwdSet:(NSString *)p
{
    pwd = p;
}

//#define FIRST_ENTRY_IS_DESCRIPTION

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view.
   
    [self.blackR setBackgroundColor:LOGO_BACK_COLOR];
    
    self.tickImage = [UIImage imageNamed:@"tick.png"];
    
    self.formatterMsg = [[NSDateFormatter alloc] init];
    [self.formatterMsg setDateFormat:@"dd/MM/yyyy HH:mm"];
    //[formatterMsg setTimeZone:[NSTimeZone timeZoneWithName:@"..."]]; //Optionally for time zone conversions
    
    self.playIcon = [UIImage imageNamed:@"play.png"];
    
    self.dateFormat = [[NSDateFormatter alloc] init];
    //[self.dateFormat setLocale:[NSLocale currentLocale]];
    [self.dateFormat setDateFormat:@"dd/MM/yyyy HH:mm"];
    //[self.dateFormat setFormatterBehavior:NSDateFormatterBehaviorDefault];
    
    [self restoreChat];

    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self
                                                                          action:@selector(dismissKeyboard)];
    [self.scrollView addGestureRecognizer:tap];
    
    [self registerForKeyboardNotifications];
    [self createChat];
    [self connectToChat];
    
    NSDictionary* viewDict = @{@"logo":self.logo, @"scroll":self.scrollView};
 
    if(self.notifyItem != nil)
    {
        self.containerView.hidden = NO;
        //--------------------------------------------------------
#if false
        DetailsViewController *innerViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"DetailsViewController"];
        innerViewController.notifyItem = self.notifyItem;
        innerViewController.parentContr = self;
        innerViewController.parentHeight = self.view.frame.size.height;
        innerViewController.parentWidth = self.view.frame.size.width;
        
        [self addChildViewController:innerViewController];
        innerViewController.view.frame = self.containerView.bounds;
        [self.containerView addSubview:innerViewController.view];
        [innerViewController didMoveToParentViewController:self];
        
        innerViewController.backgroundLogo.hidden = YES;
        innerViewController.mainView.backgroundColor = UIColor.clearColor;
        //--------------------------------------------------------
#endif
        self.blackR.hidden = self.logo.hidden = YES;
        [self.mainView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[scroll]" options:0 metrics:0 views:viewDict]];
    }
    else
    {
        self.containerView.hidden = YES;
        
        self.blackR.hidden = self.logo.hidden = NO;
        [self.mainView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:[logo][scroll]" options:0 metrics:0 views:viewDict]];
    }
    
    [self toggleLanguage];

    [self callHistory];
}

-(void)dismissKeyboard
{
    if(activeField)
        [activeField resignFirstResponder];
}

- (void)toggleLanguage
{
    NSString *table = [ViewController getLanguageSelected];
    
    //[self.addImageButton setTitle:NSLocalizedStringFromTable(@"chat aggiungi immagine", table, nil) forState:UIControlStateNormal];
    [self.historyButton setTitle:NSLocalizedStringFromTable(@"chat history", table, nil) forState:UIControlStateNormal];
    [self.sendButton setTitle:NSLocalizedStringFromTable(@"chat invia", table, nil) forState:UIControlStateNormal];
}

- (IBAction)doBackAction:(id)sender
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void) createChat
{
    historyPage = lastHistoryPage = 1;
    historyOffset = 0;
    
#if false
    scrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(0, 16, self.view.frame.size.width, self.view.frame.size.height-16)];
    
    scrollView.showsVerticalScrollIndicator=NO;
    [scrollView setBackgroundColor:[UIColor clearColor]];
    scrollView.pagingEnabled=NO;
    scrollView.directionalLockEnabled=YES;
    scrollView.bounces=YES;
    [scrollView setCanCancelContentTouches:YES];
    scrollView.userInteractionEnabled = YES;
    [self.view insertSubview:scrollView belowSubview:self.back];
    //[self.view addSubview:scrollView];
#endif
    textView = [[UITextView alloc] initWithFrame:CGRectMake(self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT + 6, self.view.frame.size.height - self.view.frame.size.height*0.08f - 2, self.view.frame.size.width - (self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT + 6)*2, self.view.frame.size.height*0.08f)];
    textView.delegate = self;
    [self.view addSubview:textView];//[scrollViewTextField insertSubview:textView aboveSubview:self.blackRibbon];
    textView.backgroundColor = [UIColor lightGrayColor];
    
    //To make the border look very close to a UITextField
    [textView.layer setBorderColor:[[[UIColor grayColor] colorWithAlphaComponent:0.5] CGColor]];
    [textView.layer setBorderWidth:1];
    
    //The rounded corner part, where you specify your view's corner radius:
    textView.layer.cornerRadius = 5;
    textView.clipsToBounds = YES;
    
    [self.sendButton.layer setBorderColor:[[[UIColor grayColor] colorWithAlphaComponent:0.5] CGColor]];
    [self.sendButton.layer setBorderWidth:1.0];
    self.sendButton.layer.cornerRadius = 5;
    self.sendButton.clipsToBounds = YES;
    
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.sendButton attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:textView attribute:NSLayoutAttributeCenterY multiplier:1.0 constant:0]];
    //[self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.sendButton attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:textView attribute:NSLayoutAttributeHeight multiplier:1.0 constant:0]];

    
    //[self.scrollViewTextField setContentSize:CGSizeMake(self.view.frame.size.width, self.view.frame.size.height)];
    
    //[TextViewController fitTextView:self.myTextView outFontSize:nil maxFontSize:20];
}

- (void)viewDidAppear:(BOOL)animated
{
    if(reconnectOnShow)
    {
        reconnectOnShow = NO;
        if(_client != nil)
            [_client connect];
    }
    
    //[self playMovie:@"http://techslides.com/demos/sample-videos/small.mp4"];
    //[self playMovie:@"http://www.ibastore.it/chat/video/full_size/by_tag/20151014185457_47130.mp4"];
    
    [self refreshChatView];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:(BOOL)animated];    // Call the super class implementation.
    // Usually calling super class implementation is done before self class implementation, but it's up to your application.
    
    if(_client != nil)
    {
        self.shouldReconnect = false;
        [_client disconnect];
    }
}

- (BOOL)pusher:(PTPusher *)pusher connectionWillAutomaticallyReconnect:(PTPusherConnection *)connection afterDelay:(NSTimeInterval)delay;
{
    return self.shouldReconnect;
}

- (void)pusher:(PTPusher *)pusher willAuthorizeChannel:(PTPusherChannel *)channel withRequest:(NSMutableURLRequest *)request
{
    //[request setValue:@"some-authentication-token" forHTTPHeaderField:@"X-MyCustom-AuthTokenHeader"];
}

- (void) connectToChat
{
    if(!self.notifyList)
        self.notifyList = [[NSMutableArray alloc] init];
    
    [self fillChannelName];
    
    // self.client is a strong instance variable of class PTPusher
    _client = [PTPusher pusherWithKey:PUSHER_APP_KEY delegate:self encrypted:YES];
    _client.authorizationURL = [NSURL URLWithString:PUSHER_AUTH_URL];
    //_client.reconnectDelay = 3600;
    
    self.shouldReconnect = YES;
    [_client connect];
    
#ifdef USE_PUSHER_PRESENCE_CHANNEL
    self.channel = [_client subscribeToPresenceChannelNamed:self.channelName delegate:self];
#else
    self.channel = [_client subscribeToPrivateChannelNamed:self.channelName];
#endif
    
    [self.channel bindToEventNamed:@"pusher:subscription_succeeded" handleWithBlock:^(PTPusherEvent *channelEvent) {
        // channelEvent.data is a NSDictionary of the JSON object received
        NSLog(@"subscription succeded");
        //TODO: aprire la scrollist coi messaggi, altrimenti back alla pagina precedente
    }];
    [self.channel bindToEventNamed:@"pusher:subscription_error" handleWithBlock:^(PTPusherEvent *channelEvent) {
        // channelEvent.data is a NSDictionary of the JSON object received
        NSLog(@"subscription error");
        //if(status == 408 || status == 503)
        {
            // se si trattasse di autenticazione fallita allora lo status sarebbe 403
            // nei casi 408 e 503 si può fare un nuovo tentativo
        }
    }];
    [self.channel bindToEventNamed:@"client-messageSent" handleWithBlock:^(PTPusherEvent *channelEvent) {
        // channelEvent.data is a NSDictionary of the JSON object received
        NSLog(@"MessageSent");
        NSString* strURL = [NSString stringWithFormat:PUSHER_GET_MSG_URL, [channelEvent.data objectForKey:@"id"]];
#if false
        NSURLRequest *site_request = [NSURLRequest requestWithURL:[NSURL URLWithString:strURL]
                                                      cachePolicy:NSURLRequestUseProtocolCachePolicy
                                                  timeoutInterval:10.0];
        
        self.downloadedData = @"";
        self.site_connection = [[NSURLConnection alloc] initWithRequest:site_request delegate:self];
#else
        HTTP *http = [[HTTP alloc] init];
        [http get:strURL withHandle:^(NSString *dataStr) {
            [self decodeResponse:dataStr triggerEvent:NO];
        }];
#endif
    }];
    [self.channel bindToEventNamed:@"client-messageRead" handleWithBlock:^(PTPusherEvent *channelEvent) {
        // channelEvent.data is a NSDictionary of the JSON object received
        NSLog(@"MessageRead");
        MessageItem *item;
        bool refreshNeeded = NO;
        for (item in self.notifyList)
        {
            if([channelEvent.data objectForKey:@"id"])
            {
                if([item.ID isEqualToString:[channelEvent.data objectForKey:@"id"]])
                {
                    if(item.statusId != READ)
                    {
                        item.statusId = READ;
                        refreshNeeded = true;
                    }
                }
            }
        }
        if(refreshNeeded)
            [self refreshChatView];
    }];
}

- (void)presenceChannel:(PTPusherPresenceChannel *)channel memberAdded:(PTPusherChannelMember *)member
{

}

- (void)presenceChannelDidSubscribe:(PTPusherPresenceChannel *)channel
{
    NSLog(@"Did Subscribe");
}

- (void)presenceChannel:(PTPusherPresenceChannel *)channel memberRemoved:(PTPusherChannelMember *)member
{

}

- (void) fillChannelName
{
    AppDelegate *delegate = [UIApplication sharedApplication].delegate;
    //TODO: fill self.username with the one provided in the login page
    
    if(/*PUSHER_USE_REGISTRATION &&*/ [LoginViewController codeGet] != nil && [LoginViewController codeGet].length > 0)
        self.channelName = [@"" stringByAppendingString:[LoginViewController codeGet]];
    else if(/*PUSHER_USE_REGISTRATION &&*/ username != nil && username.length > 0)
        self.channelName = [@"" stringByAppendingString:username];
    else if(delegate.token_string)
        self.channelName = [@"" stringByAppendingString:delegate.token_string];
#ifdef IBATEST
    else self.channelName = TEST_TOKEN;
#endif
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    NSLog(@"You entered %@", textField.text);
    [textField resignFirstResponder];
    
    return YES;
}
*/

-(BOOL)textView:(UITextView *)textView shouldChangeTextInRange:
(NSRange)range replacementText:(NSString *)text{
#if false
    if ([text isEqualToString:@"\n"]) {
        [textView resignFirstResponder];
    }
#endif
    return YES;
}

-(void)textViewDidBeginEditing:(UITextView *)textView{
    //NSLog(@"Did begin editing");
    activeField = textView;
}

-(void)textViewDidChange:(UITextView *)textView{
    //NSLog(@"Did Change");
}

BOOL testToggler;
-(void)textViewDidEndEditing:(UITextView *)textView{
    //NSLog(@"Did End editing");
    [textView resignFirstResponder];
    activeField = nil;
}

- (IBAction)doSendButton:(id)sender
{
#if false
    MessageItem *item = [[MessageItem alloc] init];
    
    item.ID = @"1234";
    item.from = testToggler?@"sender":ME_MYSELF; testToggler = !testToggler;
    item.type = @"1";
    NSString *dateStr = @"01/01/2015 12:00";
    item.message = (textView.text.length > 0)?textView.text:@"Test message";
    item.created_at = [self.dateFormat dateFromString:dateStr];
    //created_at = [item.data dateByAddingTimeInterval:timeOffset];
    item.image = nil;
    
    [self.notifyList addObject:item];
    
    [self refreshChatView];
#else
    if(textView.text.length > 0)
        [self sendMessage:textView.text];
#endif
    
    [textView resignFirstResponder];
    textView.text = @"";
}

-(BOOL)textViewShouldEndEditing:(UITextView *)textView{
    [textView resignFirstResponder];
    return YES;
}

- (void)registerForKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWasShown:)
                                                 name:UIKeyboardDidShowNotification object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillBeHidden:)
                                                 name:UIKeyboardWillHideNotification object:nil];
    
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

// Called when the UIKeyboardDidShowNotification is sent.
- (void)keyboardWasShown:(NSNotification*)aNotification
{
    NSDictionary* info = [aNotification userInfo];
    CGSize kbSize = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
#if false
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, kbSize.height, 0.0);
    self.scrollViewTextField.contentInset = contentInsets;
    self.scrollViewTextField.scrollIndicatorInsets = contentInsets;
    
    // If active text field is hidden by keyboard, scroll it so it's visible
    // Your app might not need or want this behavior.
    CGRect aRect = self.scrollViewTextField;//self.view.frame;
    aRect.size.height -= kbSize.height;
    if (!CGRectContainsPoint(aRect, activeField.frame.origin) ) {
        [self.scrollViewTextField scrollRectToVisible:activeField.frame animated:NO];
    }
#else
    oldActiveYoffset = self.view.frame.size.height - self.view.frame.size.height*0.08f;//activeField.frame.origin.y;
    activeField.frame = CGRectMake(0, oldActiveYoffset-kbSize.height, self.view.frame.size.width - (self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT)*1, activeField.frame.size.height);
#endif
    
    self.sendButton.frame = CGRectMake(self.view.frame.size.width - (self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT)*1, oldActiveYoffset-kbSize.height, (self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT + 6)*1, activeField.frame.size.height);
    self.sendButton.hidden = NO;
}

// Called when the UIKeyboardWillHideNotification is sent
- (void)keyboardWillBeHidden:(NSNotification*)aNotification
{
#if false
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    self.scrollViewTextField.contentInset = contentInsets;
    self.scrollViewTextField.scrollIndicatorInsets = contentInsets;
#else
    activeField.frame = CGRectMake(self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT + 6, oldActiveYoffset - 2, self.view.frame.size.width - (self.view.frame.size.width*TEXT_VIEW_HEIGHT_MULT + 6)*2, activeField.frame.size.height);
#endif
    
    self.sendButton.hidden = YES;
}

-(void) scrollToChatTop
{
    CGPoint topOffset = CGPointMake(0, 0);
    [self.scrollView setContentOffset:topOffset animated:YES];
}

-(void) scrollToChatEnd
{
    if(self.scrollView.contentSize.height - self.scrollView.bounds.size.height > 0)
    {
        CGPoint bottomOffset = CGPointMake(0, self.scrollView.contentSize.height - self.scrollView.bounds.size.height + self.scrollView.contentInset.bottom);
        [self.scrollView setContentOffset:bottomOffset animated:YES];
    }
}

#define MAX(a,b) ( ((a) > (b)) ? (a) : (b) )

-(void) refreshChatView
{
  @synchronized(self)
  {
    NSString *documentsDirectoryPath;
#if false
    [[self.scrollView subviews] makeObjectsPerformSelector: @selector(removeFromSuperview)];
#else
    for (UIView *v in [self.scrollView subviews]) {
        if(v != self.containerView)
            [v removeFromSuperview];
    }
#endif
    
#if true
    documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
#endif
    
    //The x and y view location coordinates for your menu items
    int headSpacer = 35;
    int footSpacer = 80;
    int insets = 10;
    int x = insets, y = headSpacer + ((self.notifyItem != nil)?self.containerView.bounds.size.height:0);
        
    //The height and width of your images
    int imageWidth = self.view.frame.size.width - insets*2;
    
    int spacer = 10;
    int counter = 0;
    int contentwidth = 320;
    int labelYgap = 10;
    
    //The content seize needs to refelect the number of items that will be added
    
    //NSString *folderPath = [NSString stringWithFormat:@"%@/gallery", [[NSBundle mainBundle] resourcePath]];
    MessageItem *item;
    
    NSSortDescriptor *sortDescriptor;
    sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"numericID"//@"created_at"
                                                  ascending:YES];
    NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    NSArray *sortedArray = /*self.notifyList =*/ [self.notifyList sortedArrayUsingDescriptors:sortDescriptors];
    
    for (item in sortedArray)
    {
        BOOL forMe = [item.from isEqualToString:ME_MYSELF];
        if([item.type isEqualToString:@"2"] || [item.type isEqualToString:@"3"])
        {
            BOOL moviewNotLoadedYet = NO;
            NSArray *myArray = [item.message componentsSeparatedByString:@"/"];
            NSString *imageName = [myArray objectAtIndex:(myArray.count-1)];
            
            UIImage *image;

            if(item.image == nil)
            {
            }
            else
            {
                image = item.image;
            }
            
            if([item.type isEqualToString:@"3"])    //movie
            {
                image = item.image = [UIImage imageNamed:@"movie.png"];
                imageWidth = self.view.frame.size.width/4;
        
                NSString *videoName = [myArray objectAtIndex:(myArray.count-1)];
                
                NSString *urlName = [[documentsDirectoryPath stringByAppendingString:@"/"] stringByAppendingString:videoName];
                
                if (![[NSFileManager defaultManager] fileExistsAtPath:urlName])
                    moviewNotLoadedYet = YES;
                else image = item.image = [self getThumbFromMovie:urlName];
            }
            else image = item.image = (item.image != nil)?item.image:[NotifiesListViewController loadImage:imageName inDirectory:documentsDirectoryPath];
            
            if(!image || moviewNotLoadedYet)
            {
                UIActivityIndicatorView *activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
                activityIndicator.alpha = 1.0;
                activityIndicator.center = CGPointMake((!forMe)?(insets):(self.view.frame.size.width - self.view.frame.size.width/2 - insets) + activityIndicator.frame.size.width/2, y + activityIndicator.frame.size.height/2);
                activityIndicator.hidesWhenStopped = YES;
                [self.scrollView addSubview:activityIndicator];
                item.loading = activityIndicator;
                [activityIndicator startAnimating];
            }
            
            if(image != nil)
            {
                imageWidth = MIN(self.view.frame.size.width/2, item.image.size.width);
                int imageHeight = imageWidth*image.size.height/image.size.width;
                imageView = [[UIImageView alloc] initWithFrame: CGRectMake(
    #ifdef IMAGE_PINCH_ZOOM_ENABLED2
                                                                           0, 0,
    #else
                                                                           x, y,
    #endif
                                                                           imageWidth, imageHeight)];
                
    #ifdef IMAGE_PINCH_ZOOM_ENABLED2
                UIScrollView *imgScrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(x, y, imageWidth, imageHeight)];
                [imgScrollView setTranslatesAutoresizingMaskIntoConstraints:NO];
                //imgScrollView = [UIColor redColor];
                imgScrollView.maximumZoomScale = 2.0;
                imgScrollView.minimumZoomScale = 1;
                imgScrollView.zoomScale = 1;
                imgScrollView.delegate = self;
    #endif
                //set image to each imageview
                imageView.image = image;
                imageView.userInteractionEnabled=YES;
                imageView.multipleTouchEnabled=YES;
                imageView.contentMode = UIViewContentModeScaleAspectFit;
                
                //add tag to each imageview in grid
                
                imageView.tag = [item.type intValue];//counter;
                //NSLog(@"%d",imageView.tag);
                
                //add tap gesture to each image view in grid
                
                UserDataTapGestureRecognizer *tapGesture = [[UserDataTapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTapGesture:)];
                tapGesture.numberOfTapsRequired = 2;
                tapGesture.userData = item;
                [imageView addGestureRecognizer:tapGesture];

                if([item.type isEqualToString:@"3"])    //movie
                {
                    CALayer *sublayer = [CALayer layer];
                    int w = imageView.bounds.size.width;
                    int h = imageView.bounds.size.height;
                    sublayer.frame = CGRectMake((w-w/2)/2, (h-h/2)/2, w/2, h/2);
                    sublayer.contents = (id) self.playIcon.CGImage;
                    sublayer.opacity = .4f;
                  
                    [imageView.layer addSublayer:sublayer];
                }
                
                //add each imageview and label to grid  scrollview
    #ifdef IMAGE_PINCH_ZOOM_ENABLED2
                CGFloat xOff = (!forMe)?(insets):(self.view.frame.size.width - imageWidth - insets);
                imgScrollView.frame = CGRectMake(xOff, imgScrollView.frame.origin.y, imageWidth, imageHeight);
                
                imgScrollView.layer.cornerRadius = 5;
                imgScrollView.clipsToBounds = YES;
                
                [imgScrollView addSubview:imageView];
                [self.scrollView addSubview:imgScrollView];
    #else
                imageView.layer.cornerRadius = 5;
                imageView.clipsToBounds = YES;
                
                [self.scrollView addSubview:imageView];
    #endif
                
    #if false
                UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(insets, y + imageHeight + labelYgap, imageWidth, 20)];
                [label setText:@"IMG:"];
                label.textAlignment = NSTextAlignmentCenter;
                [self.scrollView addSubview:label];
    #endif
                
                y += imageHeight + spacer;
            }
        }
        else
        {
            UITextView *myTextView = [[UITextView alloc] initWithFrame:CGRectMake(0, y /*+ imageHeight + labelYgap*/, self.view.frame.size.width * 0.7, 10)];
            [myTextView setText:[item.message stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]];
            myTextView.editable = myTextView.selectable = NO;
            myTextView.textAlignment = NSTextAlignmentLeft;
        
            //-------
            myTextView.font = [UIFont systemFontOfSize:self.view.bounds.size.height*16.0f/568.0f];
            //-------
            
            myTextView.backgroundColor = (!forMe)?[[UIColor whiteColor] colorWithAlphaComponent:0.6]:[[UIColor greenColor] colorWithAlphaComponent:0.2];
            
            //To make the border look very close to a UITextField
            //[myTextView.layer setBorderColor:[[[UIColor grayColor] colorWithAlphaComponent:0.5] CGColor]];
            //[myTextView.layer setBorderWidth:1.0];
            
            //The rounded corner part, where you specify your view's corner radius:
            myTextView.layer.cornerRadius = 5;
            myTextView.clipsToBounds = YES;

            myTextView.scrollEnabled = NO;
            
            CGSize sizeThatFitsTextView = [myTextView sizeThatFits:CGSizeMake(myTextView.frame.size.width, MAXFLOAT)];
            myTextView.frame = CGRectMake(0, myTextView.frame.origin.y, sizeThatFitsTextView.width, sizeThatFitsTextView.height);
            
            CGFloat fSize = myTextView.font.pointSize*.5f;
            CGFloat hSize = myTextView.font.pointSize*0.65f;
            CGFloat hLabel = 1.28f*fSize;
            CGFloat compoundedSize = MAX(sizeThatFitsTextView.width, (fSize*9.0f + hSize));
            CGFloat xOff = (!forMe)?(insets):(self.view.frame.size.width - compoundedSize - insets);

            myTextView.frame = CGRectMake(xOff, myTextView.frame.origin.y, compoundedSize, sizeThatFitsTextView.height);
            [self.scrollView addSubview:myTextView];
            
            UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(0, myTextView.frame.size.height - hLabel, myTextView.frame.size.width - hSize, hLabel)];
            label.font = [UIFont systemFontOfSize:fSize];
            label.textColor = [UIColor grayColor];
            NSString *labelStr = [self.formatterMsg stringFromDate:item.created_at];
            [label setText:labelStr];
            label.textAlignment = NSTextAlignmentRight;
            [myTextView addSubview:label];

            if(item.statusId == READ && forMe)  //if has been read
            {
                UIImageView *tickImg = [[UIImageView alloc] initWithFrame:CGRectMake(myTextView.frame.size.width - hSize, myTextView.frame.size.height - hLabel, hSize, hLabel)];
                tickImg.image = self.tickImage;
                [myTextView addSubview:tickImg];
            }
            
            y += spacer + myTextView.frame.size.height;
            
            //[TextViewController fitTextView:myTextView outFontSize:nil maxFontSize:20];
        }

        counter=counter+1;
    }
    
    //set scrollview contentsize
    
    [self.scrollView setContentSize:CGSizeMake(contentwidth, y + footSpacer)];
    
    [self storeChat];
  }
}

-(void) refreshContainerHeight:(int)height
{
    self.viewsDictionary = @{@"list":self.containerView};
    NSDictionary *metrics = @{@"height":[NSNumber numberWithInt:height]};
    
    if(self.constraint_H != nil)
        [self.scrollView removeConstraints:self.constraint_H];
    self.constraint_H = [NSLayoutConstraint constraintsWithVisualFormat:@"V:[list(height)]"
                                                                options:0
                                                                metrics:metrics
                                                                  views:self.viewsDictionary];
    [self.scrollView addConstraints:self.constraint_H];
    
    [self refreshChatView];
}

-(NSString*) getChatFile:(NotifyItem*)item
{
    NSString *userCode = [LoginViewController codeGet]?[LoginViewController codeGet]:@"unknown";
    if(!item || !item.eventCode)
        return [@"GENERAL-chat-" stringByAppendingString:userCode];
    else return [[item.eventCode stringByAppendingString:@"-chat-"] stringByAppendingString:userCode];
}

-(void) restoreChat
{
    NSString* filename = [self getChatFile:self.notifyItem];
    NSMutableArray *array = nil;
    
    NSString *error = nil;
    NSString *errorDesc = nil;
    NSPropertyListFormat format;
    NSString *plistPath;
    NSString *rootPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                              NSUserDomainMask, YES) objectAtIndex:0];
    plistPath = [rootPath stringByAppendingPathComponent:[filename stringByAppendingString:@".plist"]];
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistPath]) {
        //plistPath = [[NSBundle mainBundle] pathForResource:filename ofType:@"plist"];
    }
    
    array = [NSKeyedUnarchiver unarchiveObjectWithFile:plistPath];    //NSMutableArray *ma = [NSMutableArray arrayWithContentsOfFile:plistPath];
    if(!array)
        array = [[NSMutableArray alloc] init];
    
    for(MessageItem *item in array)
        [self downloadAssets:item];
        
    self.notifyList = array;
}

-(void) storeChat
{
    NSMutableArray *array = self.notifyList;
    NSString* filename = [self getChatFile:self.notifyItem];
    
    NSString *error = nil;
    NSString *errorDesc = nil;
    NSPropertyListFormat format;
    NSString *plistPath;
    NSString *rootPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                              NSUserDomainMask, YES) objectAtIndex:0];
    plistPath = [rootPath stringByAppendingPathComponent:[filename stringByAppendingString:@".plist"]];
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistPath]) {
        //plistPath = [[NSBundle mainBundle] pathForResource:filename ofType:@"plist"];
    }

    NSMutableArray *ma = [NSKeyedUnarchiver unarchiveObjectWithFile:plistPath];    //NSMutableArray *ma = [NSMutableArray arrayWithContentsOfFile:plistPath];
    if(!ma)
        ma = [[NSMutableArray alloc] init];
    
    for(MessageItem *item in array)
    {
        bool found = false;
        for(MessageItem *i in ma)
        {
            if([i.ID isEqualToString:item.ID])
            {
                [ma replaceObjectAtIndex:[ma indexOfObject:i] withObject:item];
                found = true;
                break;
            }
        }
        
        if(!found)
            [ma addObject:item];
    }

    [NSKeyedArchiver archiveRootObject:ma toFile:plistPath];
}

- (void)handleTapGesture:(UserDataTapGestureRecognizer *)sender
{
    if (sender.state == UIGestureRecognizerStateRecognized)
    {
        UIImageView *imageView = (UIImageView *)sender.view;
        if(imageView.tag == 2)
            [self magnifyImage:(imageView).image];
        else if(imageView.tag == 3)
        {
            MessageItem *mi = sender.userData;
            NSString *urlName = [[BASE_SERVER_URL stringByAppendingString:@"/"] stringByAppendingString:mi.message];
#if true
            NSArray *myArray = [mi.message componentsSeparatedByString:@"/"];
            NSString *filename = [myArray objectAtIndex:(myArray.count-1)];
            
            NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
            NSString *documentsDirectory = [paths objectAtIndex:0];
            urlName = [documentsDirectory stringByAppendingPathComponent:filename];
#endif
            if([[NSFileManager defaultManager] fileExistsAtPath:urlName])
                [self playMovie:urlName];
        }
    }
}

-(void) playMovie:(NSString*)urlName
{
#if false
    NSURL *url = [NSURL URLWithString:urlName/*[urlName stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]*/];
    self.player = [AVPlayer playerWithURL:url];
#else
    AVAsset *asset = [AVURLAsset URLAssetWithURL:[NSURL fileURLWithPath:urlName] options:nil];
    
#if true
    NSString *tracksKey = @"tracks";
    
    [asset loadValuesAsynchronouslyForKeys:@[tracksKey] completionHandler:
     ^{
         NSError *error;
         AVKeyValueStatus status = [asset statusOfValueForKey:tracksKey error:&error];
         
         if (status == AVKeyValueStatusLoaded) {
             
             // At this point you know the asset is ready
             AVPlayerItem *anItem = [AVPlayerItem playerItemWithAsset:asset];
             
             if(self.player)
                 [self.player removeObserver:self forKeyPath:@"status"];
             
             self.player = [AVPlayer playerWithPlayerItem:anItem];
             [self.player addObserver:self forKeyPath:@"status" options:0 context:nil];
             //[self.playerView setPlayer:self.player];
             
             [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playerItemDidReachEnd:) name:AVPlayerItemDidPlayToEndTimeNotification object:[self.player currentItem]];
         }
         else {
             // You should deal with the error appropriately.
             NSLog(@"The asset's tracks were not loaded:\n%@", [error localizedDescription]);
         }
     }];
#else
    AVPlayerItem *anItem = [AVPlayerItem playerItemWithAsset:asset];
    
    self.player = [AVPlayer playerWithPlayerItem:anItem];
    
    [self.playerView setPlayer:self.player];
    [self.player addObserver:self forKeyPath:@"status" options:0 context:nil];
    
    AVPlayerLayer *layer = [AVPlayerLayer playerLayerWithPlayer:self.player];
    self.player.actionAtItemEnd = AVPlayerActionAtItemEndNone;
    
    
    layer.frame = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    [self.view.layer addSublayer: layer];
    
    
    [self.player play];
    
    //[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playerItemDidReachEnd:) name:AVPlayerItemDidPlayToEndTimeNotification object:[self.player currentItem]];
#endif
    
#endif
}

- (IBAction)dismissPlayback:(id)sender
{
    [self closePlayer:nil];
}

- (void)observeValueForKeyPath:(NSString*) path ofObject:(id)object change:(NSDictionary*)change context:(void*)context
{
    if (self.player.status == AVPlayerStatusReadyToPlay)
    {
        [self.player play];
        self.playerView.hidden = NO;
    }
    else if (self.player.status == AVPlayerStatusFailed)
    {
        // something went wrong. player.error should contain some information
        NSLog(@"AVPlayer error %@", self.player.error);
        [self closePlayer:self.player.error];
    
    }
    else if (self.player.status == AVPlayerItemStatusUnknown)
    {
        NSLog(@"AVPlayer Unknown error");
        [self closePlayer:@"AVPlayer Unknown error"];
    }
}

-(void) closePlayer:(NSString*)error
{
    self.playerView.hidden = YES;
    //[self.player removeObserver:self forKeyPath:@"status"];
    
    if(error)
    {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Error!" message:error delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
        [alertView show];
    }
}

- (void)playerItemDidReachEnd:(NSNotification *)notification
{
    //[self.player seekToTime:kCMTimeZero]; //this resets to the head of the movie
    [self closePlayer:nil];
}

- (UIView*)viewForZoomingInScrollView:(UIScrollView *)scrollView
{
    return [scrollView subviews][0];
}

- (void)magnifyImage:(UIImage*)image
{
    UIStoryboard *storyboard = self.storyboard;
#if false
    MagnifyViewController *modalViewController = [storyboard instantiateViewControllerWithIdentifier:@"MagnifyViewController"];
    
    modalViewController.selectedImage = image;
    
    [ViewController setPresentationStyleForSelfController:self presentingController:modalViewController];
    [self presentViewController:modalViewController animated:NO completion:nil];
#endif
}

- (MessageItem*) parseMessageItem:(MessageItem*)item fromArray:(NSDictionary*)itemArray
{
    item.ID = ([[itemArray objectForKey:@"id"] isKindOfClass:[NSNumber class]])?[[itemArray objectForKey:@"id"] stringValue] : [itemArray objectForKey:@"id"];
    item.numericID = [item.ID intValue];
    item.from = [itemArray objectForKey:@"from"];
    item.type = [itemArray objectForKey:@"type_id"];
    NSString *dateStr = [itemArray objectForKey:@"created_at"];
    item.message = [itemArray objectForKey:@"message"];
    item.created_at = [self.dateFormat dateFromString:dateStr];
    //created_at = [item.data dateByAddingTimeInterval:timeOffset];
    item.image = nil;
    if([itemArray objectForKey:@"status_id"])
        item.statusId = [[itemArray objectForKey:@"status_id"] intValue];
    else item.statusId = SENT;
    if([itemArray objectForKey:@"event_code"])
        item.eventCode = [itemArray objectForKey:@"event_code"];
    
    [self downloadAssets:item];
    
    return item;
}

-(void) downloadAssets:(MessageItem*)item
{
    if([item.type isEqualToString:@"2"])
    {
        NSString *documentsDirectoryPath;
#if true
        documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
#endif
        NSArray *myArray = [item.message componentsSeparatedByString:@"/"];
        NSString *imageName = [myArray objectAtIndex:(myArray.count-1)];
        
        NSString *urlName = [[BASE_SERVER_URL stringByAppendingString:@"/"] stringByAppendingString:item.message];
        UIImage *image = [NotifiesListViewController loadImage:imageName inDirectory:documentsDirectoryPath];
        if(image)
            item.image = image;
        else
        {
#if true
            //these are async
            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
                item.image = [NotifiesListViewController getImageFromURL:urlName];
                if(item.image)
                    [NotifiesListViewController saveImage:item.image withFileName:imageName inDirectory:documentsDirectoryPath];
                
                dispatch_async(dispatch_get_main_queue(), ^{
                    if(item.loading)
                        [item.loading stopAnimating];
                    [self refreshChatView];
                });
            });
#else
            item.image = [NotifiesListViewController getImageFromURL:urlName];
            if(item.image)
                [NotifiesListViewController saveImage:item.image withFileName:imageName inDirectory:documentsDirectoryPath];
#endif
        }
    }
    else if([item.type isEqualToString:@"3"])
    {
        NSString *documentsDirectoryPath;
#if true
        documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
#endif
        NSArray *myArray = [item.message componentsSeparatedByString:@"/"];
        NSString *videoName = [myArray objectAtIndex:(myArray.count-1)];
        
        NSString *urlName = [[documentsDirectoryPath stringByAppendingString:@"/"] stringByAppendingString:videoName];
        
        if (![[NSFileManager defaultManager] fileExistsAtPath:urlName])
        {
            [self downloadVideoAndSave:[[BASE_SERVER_URL stringByAppendingString:@"/"] stringByAppendingString:item.message] withFilename:videoName withItem:item];
        }
    }
}

-(void)downloadVideoAndSave :(NSString*)videoUrl withFilename:(NSString*)filename withItem:(MessageItem*)item
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        
        NSData *yourVideoData=[NSData dataWithContentsOfURL:[NSURL URLWithString:videoUrl]];
        
        if (yourVideoData) {
            NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
            NSString *documentsDirectory = [paths objectAtIndex:0];
            
            NSString  *filePath = [NSString stringWithFormat:@"%@/%@", documentsDirectory, filename];
            
            if([yourVideoData writeToFile:filePath atomically:YES])
            {
                NSLog(@"write successfull");
            }
            else{
                NSLog(@"write failed");
            }
        }
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if(item.loading)
                [item.loading stopAnimating];
            [self refreshChatView];
        });
    });
}

-(UIImage*) getThumbFromMovie:(NSString*)filePath
{
    NSError *err = NULL;
    AVAsset *asset = [AVURLAsset URLAssetWithURL:[NSURL fileURLWithPath:filePath] options:nil];
    
    AVAssetImageGenerator *generator = [[AVAssetImageGenerator alloc] initWithAsset:asset];
    generator.appliesPreferredTrackTransform=TRUE;
    //[asset release];
    CMTime thumbTime = CMTimeMakeWithSeconds(0, 5);
    
    CGImageRef oneRef = [generator copyCGImageAtTime:thumbTime actualTime:NULL error:&err];
    UIImage *img = [[UIImage alloc] initWithCGImage:oneRef];
    
    return img;
}

//this is for history GET
- (void)didFinishDownload:(NSString *)data
{
    NSDictionary* contentArray;
    NSError* error;
    NSData *encData = [data dataUsingEncoding:NSUTF8StringEncoding];
    contentArray = [NSJSONSerialization JSONObjectWithData:encData options:kNilOptions error:&error];
    if (error != nil)
    {
        NSLog(@"Finish Download error: %@", [error localizedDescription]);
        return;
    }

    if(contentArray != nil)
    {
        for(NSDictionary *itemArray in [contentArray objectForKey:@"messages"])
        {
            //NSArray *itemArray = [line componentsSeparatedByString:@";"];
            MessageItem *item = [[MessageItem alloc] init];
            item = [self parseMessageItem:item fromArray:itemArray];
            NSUInteger index = [self.notifyList indexOfObjectPassingTest:^BOOL(id obj, NSUInteger idx, BOOL *stop) {
                return ([[(MessageItem *)obj ID] isEqualToString:item.ID]);
            }];
            if([self checkRightChatChannel:item])
            {
                if(index == NSNotFound)
                    [self.notifyList addObject:item];
                else [self.notifyList replaceObjectAtIndex:index withObject:item];
            }
        }
        
        historyPage = [[contentArray objectForKey:@"page"] intValue];
        lastHistoryPage = [[contentArray objectForKey:@"last_page"] intValue];
        
        if(lastHistoryPage > historyPage)
            historyPage++;
        else [self.historyButton setEnabled: NO];
        
        [self refreshChatView];
        if(self.notifyList && self.notifyList.count > 0)
            [self scrollToChatEnd];
    }
}

- (void)didFail:(NSString *)failedMessage {}

- (void) callHistory
{
    HTTP *http = [[HTTP alloc] init];
    http.delegate = self;
    int hPage = 1 + self.notifyList.count/HISTORY_PAGE_SIZE;  //historyPage + (int)historyOffset/HISTORY_PAGE_SIZE;
    int hSize = HISTORY_PAGE_SIZE;  //(HISTORY_PAGE_SIZE + (int)historyOffset%HISTORY_PAGE_SIZE);
    NSString *url;
    int msgID = 0;
    
    if(self.notifyList && self.notifyList.count > 0)
    {
        NSMutableArray *array = [NSMutableArray array];
        for(MessageItem *mi in self.notifyList)
        {
            if(mi.statusId  != READ) {
                [array addObject:mi];
            }
        }
        
        if(array.count > 0)
        {
            NSSortDescriptor *sortDescriptor;
            sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"numericID"/*@"created_at"*/ ascending:NO];
            NSArray *sortDesc = [NSArray arrayWithObjects:sortDescriptor, nil];
            NSArray *sortedUnread = [array sortedArrayUsingDescriptors:sortDesc];
        
            msgID = ((MessageItem*)[sortedUnread objectAtIndex:(sortedUnread.count - 1)]).numericID;
        }
        else
        {
            NSSortDescriptor *sortDescriptor1, *sortDescriptor2;
            sortDescriptor1 = [[NSSortDescriptor alloc] initWithKey:@"numericID"/*@"created_at"*/ ascending:YES];
            //sortDescriptor2 = [[NSSortDescriptor alloc] initWithKey:@"statusId" ascending:YES];
            NSArray *sortDescriptors = [NSArray arrayWithObjects:sortDescriptor1, /*sortDescriptor2,*/ nil];
            NSArray *sortedArray = /*self.notifyList =*/ [self.notifyList sortedArrayUsingDescriptors:sortDescriptors];
        
            msgID = ((MessageItem*)[sortedArray objectAtIndex:(sortedArray.count - 1)]).numericID;
        }
    }
    
    if(self.notifyItem == nil)
    {
        //url = [NSString stringWithFormat:CHAT_HISTORY_URL, [NSString stringWithFormat:@"%d", hPage]/*page*/, [NSString stringWithFormat:@"%d",hSize]/*page_size*/];
        url = [NSString stringWithFormat:CHAT_HISTORY_URL, [NSString stringWithFormat:@"%d", msgID]];
    }
    else
    {
        //url = [NSString stringWithFormat:CHAT_HISTORY_SPECIFIC_URL, self.notifyItem.eventCode, [NSString stringWithFormat:@"%d", hPage]/*page*/, [NSString stringWithFormat:@"%d",hSize]/*page_size*/];
        url = [NSString stringWithFormat:CHAT_HISTORY_SPECIFIC_URL, self.notifyItem.eventCode, [NSString stringWithFormat:@"%d", msgID]];
    }
    
    [http get:url];
}

- (IBAction)doHistory:(id)sender
{
    [self callHistory];
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

- (IBAction)doAddImage:(id)sender
{
    UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
    imagePicker.delegate = self;
    imagePicker.allowsEditing = YES;
    imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    imagePicker.mediaTypes = [[NSArray alloc] initWithObjects:(NSString*)kUTTypeImage, (NSString *)kUTTypeMovie, nil];
    [self presentModalViewController:imagePicker animated:YES];
    
    reconnectOnShow = YES;
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    NSString *mediaType = [info objectForKey:UIImagePickerControllerMediaType];
    
    NSLog(@"Type is: %@", mediaType);
    
    if([mediaType isEqualToString:(NSString*)kUTTypeImage])
    {
        UIImage *image = [info objectForKey:@"UIImagePickerControllerOriginalImage"/*UIImagePickerControllerEditedImage*/];
        [self sendImage:image];
    }
    else if([mediaType isEqualToString:(NSString*)kUTTypeVideo]
            || [mediaType isEqualToString:(NSString*)kUTTypeMovie]
            || [mediaType isEqualToString:(NSString*)kUTTypeQuickTimeMovie]
            || [mediaType isEqualToString:(NSString*)kUTTypeMPEG]
            || [mediaType isEqualToString:(NSString*)kUTTypeMPEG2Video]
            || [mediaType isEqualToString:(NSString*)kUTTypeMPEG4]
            || [mediaType isEqualToString:(NSString*)kUTTypeAVIMovie]
            )
    {
        NSURL *videoUrl=(NSURL*)[info objectForKey:UIImagePickerControllerMediaURL];
        NSString *moviePath = [videoUrl path];
        
#if false
        if (UIVideoAtPathIsCompatibleWithSavedPhotosAlbum (moviePath)) {
            UISaveVideoAtPathToSavedPhotosAlbum (moviePath, nil, nil, nil);
        }
#endif
        [self sendVideo:moviePath];
    }
    
    [picker dismissModalViewControllerAnimated:YES];
}

- (IBAction)openWebpage:(id)sender {
    NSString *str = HOMEPAGE;//[(UIButton *)sender currentTitle];
    [InfoPageViewController openWeb:str];
}

- (void) sendMessage:(NSString*)msg
{
    if(msg == nil || [msg length] == 0)
        return;
    
    NSString* strURL = PUSHER_SEND_MSG_URL;
    NSMutableURLRequest *site_request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:strURL]
                                                  cachePolicy:NSURLRequestUseProtocolCachePolicy
                                              timeoutInterval:10.0];
    site_request.HTTPMethod = @"POST";
    
    NSDictionary* bodyParameters = @{
                                     @"type_id": @"1",
                                     @"content": msg,
                                     @"event_code": ((self.notifyItem != nil)?self.notifyItem.eventCode:@"")
                                     };
    site_request.HTTPBody = [NSStringFromQueryParameters(bodyParameters) dataUsingEncoding:NSUTF8StringEncoding];

    self.downloadedData = @"";
    self.site_connection = [NSURLConnection connectionWithRequest:site_request delegate:self];
    [self.site_connection start];
}

- (void) sendImage:(UIImage*)img
{
    NSString* strURL = PUSHER_SEND_MSG_URL;
    NSMutableURLRequest *site_request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:strURL]
                                                                cachePolicy:NSURLRequestUseProtocolCachePolicy
                                                            timeoutInterval:10.0];
    site_request.HTTPMethod = @"POST";
    
    NSMutableData *body = [NSMutableData data];
    NSString *boundary = @"------WebKitFormBoundarykemRODuupBakkg3G";
    NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
    [site_request addValue:contentType forHTTPHeaderField: @"Content-Type"];
    
    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"Content-Disposition: form-data; name=\"content\"; filename=\"image.jpg\"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"Content-Type: image/jpeg\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"Content-Transfer-Encoding: binary\r\n\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[NSData dataWithData:UIImageJPEGRepresentation(img, 0.9)]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    
    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"type_id\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"2" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];

    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"event_code\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[((self.notifyItem != nil)?self.notifyItem.eventCode:@"") dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];

    site_request.HTTPBody = body;
    //NSLog([[NSString alloc]initWithData:body encoding:NSUTF8StringEncoding]);
    self.downloadedData = @"";
    self.site_connection = [NSURLConnection connectionWithRequest:site_request delegate:self];
    [self.site_connection start];
}

- (void) sendVideo:(NSString*)moviePath
{
    NSString* strURL = PUSHER_SEND_MSG_URL;
    NSMutableURLRequest *site_request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:strURL]
                                                                cachePolicy:NSURLRequestUseProtocolCachePolicy
                                                            timeoutInterval:10.0];
    site_request.HTTPMethod = @"POST";
    
    NSMutableData *body = [NSMutableData data];
    NSString *boundary = @"------WebKitFormBoundarykemRODuupBakkg3G";
    NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
    [site_request addValue:contentType forHTTPHeaderField: @"Content-Type"];
    
    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"Content-Disposition: form-data; name=\"content\"; filename=\"image.jpg\"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[[NSString stringWithFormat:@"Content-Type: video/%@\r\n", moviePath.pathExtension] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"Content-Transfer-Encoding: binary\r\n\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[NSData dataWithContentsOfFile:moviePath]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    
    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"type_id\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"3" dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];

    [body appendData:[[NSString stringWithFormat:@"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"event_code\"\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[((self.notifyItem != nil)?self.notifyItem.eventCode:@"") dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:[@"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];

    site_request.HTTPBody = body;
    //NSLog([[NSString alloc]initWithData:body encoding:NSUTF8StringEncoding]);
    self.downloadedData = @"";
    self.site_connection = [NSURLConnection connectionWithRequest:site_request delegate:self];
    [self.site_connection start];
}

- (void)connection:(NSURLConnection *)connection
  didFailWithError:(NSError *)error
{
    /*
     if(self.site_connection != nil)
     self.site_connection = nil;
     */
    //[self.activityIndicator stopAnimating];
    
    //TODO: inform the user with alert
    NSLog(@"Connection failed! Error - %@ %@",
          [error localizedDescription],
          [[error userInfo] objectForKey:NSURLErrorFailingURLStringErrorKey]);
}

-(void)connection:(NSURLConnection *)site_connection didReceiveData:(NSData *)data
{
    self.downloadedData = [self.downloadedData stringByAppendingString:[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]];
}

-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    int code = [(NSHTTPURLResponse *)response statusCode];
    if (code == 404)
    {
        // website not found
        // do your stuff according to your need
    }
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    //[self.activityIndicator stopAnimating];
    [self decodeResponse:self.downloadedData triggerEvent:YES];
}

-(bool)checkRightChatChannel:(MessageItem*)item
{
    if(self.notifyItem == nil && (item.eventCode == nil || [item.eventCode isEqualToString:@""]))
       return true;
    else if(self.notifyItem != nil && [item.eventCode isEqualToString:self.notifyItem.eventCode])
        return true;
    else return false;
}

- (void)decodeResponse:(NSString*)data triggerEvent:(BOOL)trigEvent
{
    //NSLog(data);
    NSDictionary* contentArray;
    NSError* error;
    NSData *encData = [data dataUsingEncoding:NSUTF8StringEncoding];
    contentArray = [NSJSONSerialization JSONObjectWithData:encData options:kNilOptions error:&error];
    if (error != nil)
    {
        NSLog(@"Decode Response error: %@", [error localizedDescription]);
        return;
    }

    if(contentArray != nil)
    {
        //for(NSDictionary *itemArray in contentArray)
        {
            //NSArray *itemArray = [line componentsSeparatedByString:@";"];
            MessageItem *item = [[MessageItem alloc] init];
            
            item = [self parseMessageItem:item fromArray:contentArray];
            
            if([self checkRightChatChannel:item])
            {
                [self.notifyList addObject:item];
                ++historyOffset;//enable this if you want to track the history discarding the messages already arrived
                
                if([item.from isEqualToString:ME_MYSELF])
                {
                    if(trigEvent)
                        [self.channel triggerEventNamed:@"messageSent" data:@{@"id": item.ID}];
                }
            }
        }
        
        [self refreshChatView];
        if(self.notifyList && self.notifyList.count > 0)
            [self scrollToChatEnd];
    }
}

/**
 This creates a new query parameters string from the given NSDictionary. For
 example, if the input is @{@"day":@"Tuesday", @"month":@"January"}, the output
 string will be @"day=Tuesday&month=January".
 @param queryParameters The input dictionary.
 @return The created parameters string.
 */
static NSString* NSStringFromQueryParameters(NSDictionary* queryParameters)
{
    NSMutableArray* parts = [NSMutableArray array];
    [queryParameters enumerateKeysAndObjectsUsingBlock:^(id key, id value, BOOL *stop) {
        NSString *part = [NSString stringWithFormat: @"%@=%@",
                          [key stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding],
                          [value stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]
                          ];
        [parts addObject:part];
    }];
    return [parts componentsJoinedByString: @"&"];
}

//////////////////////////////////
#pragma mark - Pusher Delegate Connection
//////////////////////////////////

- (void)pusher:(PTPusher *)pusher connectionDidConnect:(PTPusherConnection *)connection
{
    NSLog(@"[Pusher] connected to %@", [connection.URL absoluteString]);
}

//-------- handling disconnection -------------
- (void)pusher:(PTPusher *)pusher connection:(PTPusherConnection *)connection failedWithError:(NSError *)error
{
    if (error) {
        NSLog(@"[Pusher] connection failed: %@", [error localizedDescription]);
    } else {
        NSLog(@"[Pusher] connection failed");
    }
    
    [self handleDisconnectionWithError:error];
}

- (void)pusher:(PTPusher *)pusher connection:(PTPusherConnection *)connection didDisconnectWithError:(NSError *)error willAttemptReconnect:(BOOL)willAttemptReconnect
{
    if (error) {
        NSLog(@"[Pusher] didDisconnectWithError: %@ willAttemptReconnect: %@", [error localizedDescription], (willAttemptReconnect ? @"YES" : @"NO"));
    } else {
        NSLog(@"[Pusher] disconnected");
    }
 
    BOOL safeDisconnect = NO;
#if false
    if([error objectForKey:@"reason"])
        if([error objectForKey:@"reason"] isEqualToString @"")
            safeDisconnect = YES;
#else
    if(error.code == 1001)
        safeDisconnect = YES;
#endif
    
    if (!willAttemptReconnect && !safeDisconnect) {
        [self handleDisconnectionWithError:error];
    }
}

- (void)handleDisconnectionWithError:(NSError *)error
{
    Reachability *reachability = [Reachability reachabilityWithHostName:_client.connection.URL.host];

    if (error && [error.domain isEqualToString:PTPusherFatalErrorDomain]) {
        NSLog(@"FATAL PUSHER ERROR, COULD NOT CONNECT! %@", error);
    }
    else {
        if([InfoPageViewController checkConnectivity]) {
            // we do have reachability so let's wait for a set delay before trying again
            [_client performSelector:@selector(connect) withObject:nil afterDelay:5];
        }
        else {
            // we need to wait for reachability to change
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(_reachabilityChanged:)
                                                         name:kReachabilityChangedNotification
                                                       object:reachability];
            
            [reachability startNotifier];
        }
    }
}

- (void)_reachabilityChanged:(NSNotification *)note
{
    Reachability *reachability = [note object];
    
    if([InfoPageViewController checkConnectivity]) {
        // we're reachable, we can try and reconnect, otherwise keep waiting
        [_client connect];
        
        // stop watching for reachability changes
        [reachability stopNotifier];
        
        [[NSNotificationCenter defaultCenter]
         removeObserver:self 
         name:kReachabilityChangedNotification 
         object:reachability];
    }
}

@end
