//
//  HTTP.m
//  template
//
//  Created by Andrea della Porta on 19/08/15.
//  Copyright (c) 2015 Ermes Italia. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "HTTP.h"

#define ENABLE_LOG  NO

@implementation HTTP

@synthesize receivedData;

- init {
    if ((self = [super init])) {
        
    }
    self.httpHandle = nil;
    
    return self;
}

- (void)dealloc {
    //[super dealloc];
}


- (void)setDelegate:(id)val
{
    delegate = val;
}

- (id)delegate
{
    return delegate;
}

- (void)get: (NSString *)urlString {
    
    if(ENABLE_LOG) NSLog ( @"GET: %@", urlString );
    
    self.receivedData = [[NSMutableData alloc] init];
    
    NSURLRequest *request = [[NSURLRequest alloc]
                             initWithURL: [NSURL URLWithString:urlString]
                             cachePolicy: NSURLRequestReloadIgnoringLocalCacheData
                             timeoutInterval: 10
                             ];
    
    NSURLConnection *connection = [[NSURLConnection alloc]
                                   initWithRequest:request
                                   delegate:self
                                   startImmediately:YES];
    if(!connection) {
        if(ENABLE_LOG) NSLog(@"connection failed :(");
    } else {
        if(ENABLE_LOG) NSLog(@"connection succeeded  :)");
        
    }
    
    //[connection release];
    //[request release];
    //[receivedData release];
}

- (void)get: (NSString *)urlString withHandle:(httpHandlePtr)handle
{
    self.httpHandle = handle;
    [self get:urlString];
}

- (void)post: (NSString *)urlString withParams:(NSDictionary*)bodyParameters
{
    NSString* strURL = urlString;
    NSMutableURLRequest *site_request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:strURL]
                                                                cachePolicy:NSURLRequestUseProtocolCachePolicy
                                                            timeoutInterval:10.0];
    site_request.HTTPMethod = @"POST";
    
    self.receivedData = [[NSMutableData alloc] init];
    
    site_request.HTTPBody = [NSStringFromQueryParameters(bodyParameters) dataUsingEncoding:NSUTF8StringEncoding];
    
    NSURLConnection *site_connection = [NSURLConnection connectionWithRequest:site_request delegate:self];
    [site_connection start];
}

/**
 This creates a new query parameters string from the given NSDictionary. For
 example, if the input is @{@"day":@"Tuesday", @"month":@"January"}, the output
 string will be @"day=Tuesday&month=January".
 @param queryParameters The input dictionary.
 @return The created parameters string.
 */
static NSString* NSStringFromQueryParameters(NSDictionary* queryParameters)
{
    NSMutableArray* parts = [NSMutableArray array];
    [queryParameters enumerateKeysAndObjectsUsingBlock:^(id key, id value, BOOL *stop) {
        NSString *part = [NSString stringWithFormat: @"%@=%@",
                          [key stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding],
                          [value stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]
                          ];
        [parts addObject:part];
    }];
    return [parts componentsJoinedByString: @"&"];
}

// ====================
// Callbacks
// ====================

#pragma mark NSURLConnection delegate methods
- (NSURLRequest *)connection:(NSURLConnection *)connection
             willSendRequest:(NSURLRequest *)request
            redirectResponse:(NSURLResponse *)redirectResponse {
    if(ENABLE_LOG) NSLog(@"Connection received data, retain count");
    return request;
}


- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    if(ENABLE_LOG) NSLog(@"Received response: %@", response);
    
    [receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    if(ENABLE_LOG) NSLog(@"Received %d bytes of data", [data length]);
    //NSLog([[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]);
    [receivedData appendData:data];
    if(ENABLE_LOG) NSLog(@"Received data is now %d bytes", [receivedData length]);
    
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    if(ENABLE_LOG) NSLog(@"Error receiving response: %@", error);
    //[[NSAlert alertWithError:error] runModal];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    // Once this method is invoked, "responseData" contains the complete result
    if(ENABLE_LOG) NSLog(@"Succeeded! Received %d bytes of data", [receivedData length]);
    
    NSString *dataStr=[[NSString alloc] initWithData:receivedData encoding:NSASCIIStringEncoding];
    if(ENABLE_LOG) NSLog(@"Succeeded! Received %@ bytes of data", dataStr);
    
    if(self.httpHandle)
        self.httpHandle(dataStr);
    else if ([delegate respondsToSelector:@selector(didFinishDownload:)]) {
        if(ENABLE_LOG) NSLog(@"Calling the delegate");
        //NSString* dataAsString = [[[NSString alloc] initWithData:receivedData encoding:NSUTF8StringEncoding] autorelease];
        [delegate performSelector:@selector(didFinishDownload:) withObject: dataStr];
    }
    
    //[dataStr release];
}


@end