//
//  InfoPageViewController.m
//
//
//  Created by Andrea della Porta on 11/03/15.
//  Copyright (c) 2015 Ermes Italia. All rights reserved.
//

#import "InfoPageViewController.h"
#import <MapKit/MapKit.h>
#import "CustomData.h"
#import "ViewController.h"

@interface InfoPageViewController ()

-(void)sendMail:(NSString*)recipient withSubject:(NSString*)subject withBody:(NSString*)body;
@property NSString *phoneNumberToCall;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property NSDictionary *viewsDictionary;
@property NSArray *constraint_POS_V;
@property (weak, nonatomic) IBOutlet UIView *addressView;
@property (weak, nonatomic) IBOutlet UIView *socialContainerView;
@property (weak, nonatomic) IBOutlet UILabel *blackribbon;

@end

@implementation InfoPageViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    [self.blackribbon setBackgroundColor:LOGO_BACK_COLOR];
    
    [self.addressBtn setTitle:ADDRESS forState:UIControlStateNormal];
    self.addressLabel.text = ADDRESS;
    [self.phoneBtn setTitle:PHONE_NUMBER forState:UIControlStateNormal];
    self.phoneLabel.text = PHONE_NUMBER;
    [self.mobileBtn setTitle:MOBILE_NUMBER forState:UIControlStateNormal];
    self.mobileLabel.text = MOBILE_NUMBER;
    [self.mailBtn setTitle:MAIL forState:UIControlStateNormal];
    self.mailLabel.text = MAIL;
    [self.webBtn setTitle:HOMEPAGE forState:UIControlStateNormal];
    self.webLabel.text = HOMEPAGE;
    
    self.scrollView.contentSize = CGSizeMake(self.view.frame.size.width, self.view.frame.size.height*8.0f/7.0f + (SHOW_VIDEO_PAGE?(self.view.frame.size.height*0.35f):0));
    
    self.viewsDictionary = @{@"addressView":self.addressView, @"socialContainerView":self.socialContainerView, @"scrollView":self.scrollView};
    
    if(self.constraint_POS_V != nil)
        [self.scrollView removeConstraints:self.constraint_POS_V];
    if(!SHOW_VIDEO_PAGE)
    {
        self.socialContainerView.hidden = YES;
        self.constraint_POS_V = [NSLayoutConstraint constraintsWithVisualFormat:@"V:[scrollView]-20-[addressView]"
                                                                    options:0
                                                                    metrics:nil
                                                                      views:self.viewsDictionary];
    }
    else
    {
        self.socialContainerView.hidden = NO;
        self.constraint_POS_V = [NSLayoutConstraint constraintsWithVisualFormat:@"V:[socialContainerView]-20-[addressView]"
                                                                                 options:0
                                                                                 metrics:nil
                                                                                   views:self.viewsDictionary];
    }
    [self.scrollView addConstraints:self.constraint_POS_V];
    
    [self toggleLanguage];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)toggleLanguage
{
    NSString *table = [ViewController getLanguageSelected];
    
    self.addressLbl.text = NSLocalizedStringFromTable(@"info indirizzo", table, nil);
    self.phoneLbl.text = NSLocalizedStringFromTable(@"info telefono fisso", table, nil);
    self.mobileLbl.text = NSLocalizedStringFromTable(@"info mobile", table, nil);
    self.mailLbl.text = NSLocalizedStringFromTable(@"info mail", table, nil);
    self.webLbl.text = NSLocalizedStringFromTable(@"info web", table, nil);
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

+(BOOL)checkConnectivity
{
    Reachability *networkReachability = [Reachability reachabilityForInternetConnection];
    NetworkStatus networkStatus = [networkReachability currentReachabilityStatus];
    if (networkStatus == NotReachable) {
        return NO;
    } else {
        return YES;
    }
}

-(void)sendMail:(NSString*)recipient withSubject:(NSString*)subject withBody:(NSString*)body
{
    if ([MFMailComposeViewController canSendMail])
    {
        mailComposer = [[MFMailComposeViewController alloc]init];
        mailComposer.mailComposeDelegate = self;
        [mailComposer setToRecipients:@[recipient]];
        [mailComposer setSubject:subject];
        [mailComposer setMessageBody:body isHTML:YES];
        [self presentViewController:mailComposer animated:YES completion:NULL];
    }
    else
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Failure"
                                                        message:@"Your device doesn't support the mail composer sheet"
                                                       delegate:nil
                                              cancelButtonTitle:@"OK"
                                              otherButtonTitles: nil];
        [alert show];
    }
}

- (void) mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    switch (result)
    {
        case MFMailComposeResultCancelled:
            NSLog(@"Mail cancelled");
            break;
        case MFMailComposeResultSaved:
            NSLog(@"Mail saved");
            break;
        case MFMailComposeResultSent:
            NSLog(@"Mail sent");
            break;
        case MFMailComposeResultFailed:
            NSLog(@"Mail sent failure: %@", [error localizedDescription]);
            break;
        default:
            break;
    }
    
    // Close the Mail Interface
    [self dismissViewControllerAnimated:YES completion:NULL];
}

-(void) openMap:(NSString*)placeholder latitude:(float)lat longitude:(float)lon
{
    Class mapItemClass = [MKMapItem class];
    if (mapItemClass && [mapItemClass respondsToSelector:@selector(openMapsWithItems:launchOptions:)])
    {
        // Create an MKMapItem to pass to the Maps app
        CLLocationCoordinate2D coordinate =
        CLLocationCoordinate2DMake(lat, lon);
        MKPlacemark *placemark = [[MKPlacemark alloc] initWithCoordinate:coordinate
                                                   addressDictionary:nil];
        MKMapItem *mapItem = [[MKMapItem alloc] initWithPlacemark:placemark];
        [mapItem setName:placeholder];
        // Pass the map item to the Maps app
        [mapItem openInMapsWithLaunchOptions:nil];
    }
}

+(void)callNumber:(NSString*)number {
    NSURL *callUrl = [NSURL URLWithString:[@"tel://" stringByAppendingString:[number stringByReplacingOccurrencesOfString:@" " withString:@"" ]]];

    if([[UIApplication sharedApplication] canOpenURL:callUrl]) {
        [[UIApplication sharedApplication] openURL:callUrl];
    }
    else {
        UIAlertView *calert = [[UIAlertView alloc]initWithTitle:@"Alert" message:@"Call facility is not available!!!" delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil, nil];
        [calert show];
    }
}

- (void) showPhoneAlert:(NSString *)number
{
    self.phoneNumberToCall = number;
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Telefonata" message:[@"Vuoi chiamare il numero " stringByAppendingString:number] delegate:self cancelButtonTitle:@"Esci" otherButtonTitles:@"Chiama", nil];
    [alert show];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == 0)
    {
        //Exit
    }
    if (buttonIndex == 1)
    {
        [InfoPageViewController callNumber:self.phoneNumberToCall];
    }
}

+(void)openWeb:(NSString*)url {
    if(![url hasPrefix:@"http"])
        url = [@"http://" stringByAppendingString:url];
    NSURL *URL = [NSURL URLWithString:[url stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
    [[UIApplication sharedApplication] openURL:URL];
}

- (IBAction)callPhone:(id)sender {
    //[(UIButton *)sender currentTitle];
    [self showPhoneAlert:PHONE_NUMBER];
}

- (IBAction)callMobile:(id)sender {
    //[(UIButton *)sender currentTitle];
    [self showPhoneAlert:MOBILE_NUMBER];
}

- (IBAction)sendMail:(id)sender {
    NSString* recp = MAIL;//[(UIButton *)sender currentTitle];
    [self sendMail:recp withSubject:@"" withBody:@""];
}

- (IBAction)goToWebpage:(id)sender {
    NSString *str = HOMEPAGE;//[(UIButton *)sender currentTitle];
    [InfoPageViewController openWeb:str];
}

- (IBAction)showMap:(id)sender {
    [self openMap:MAP_PLACEHOLDER latitude:LAT longitude:LON];
}

@end
